/**CFile****************************************************************

  FileName    [ntki.c]

  PackageName [MVSIS 2.0: Multi-valued logic synthesis system.]

  Synopsis    [Command file of the network package.]

  Author      [MVSIS Group]
  
  Affiliation [UC Berkeley]

  Date        [Ver. 1.0. Started - February 1, 2003.]

  Revision    [$Id: ntki.c,v 1.5 2003/05/27 23:14:29 alanmi Exp $]

***********************************************************************/

#include "ntkInt.h"
#include "ntkiInt.h"
#include "sh.h"
#include "wn.h"

////////////////////////////////////////////////////////////////////////
///                        DECLARATIONS                              ///
////////////////////////////////////////////////////////////////////////
 
static int Ntk_CommandPrint            ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandPrintIo          ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandPrintStats       ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandPrintRange       ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandPrintValue       ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandPrintFactor      ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandPrintLevel       ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandPrintNd          ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandPrintSpec        ( Mv_Frame_t * pMvsis, int argc, char ** argv );

static int Ntk_CommandPrintClauses     ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandPrintSat         ( Mv_Frame_t * pMvsis, int argc, char ** argv );

static int Ntk_CommandResetName        ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandRename           ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandChangeNameMode   ( Mv_Frame_t * pMvsis, int argc, char ** argv );

static int Ntk_CommandNetworkPower     ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandNetworkNDize     ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandNetworkDize      ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandNetworkReorder   ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandNetworkCopy      ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandNetworkFree      ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandNetworkDefault   ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandNetworkWindow    ( Mv_Frame_t * pMvsis, int argc, char ** argv );

static int Ntk_CommandNetworkSweep     ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandNetworkEliminate ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandNetworkCollapse  ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandNetworkMerge     ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandNetworkFxu       ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandNetworkDecomp    ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandNetworkStrash    ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandNetworkResetDef  ( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandNetworkFfResetDef( Mv_Frame_t * pMvsis, int argc, char ** argv );
static int Ntk_CommandNetworkResub     ( Mv_Frame_t * pMvsis, int argc, char ** argv );

static int Ntk_CommandNetworkVerify    ( Mv_Frame_t * pMvsis, int argc, char ** argv );


////////////////////////////////////////////////////////////////////////
///                     FUNCTION DEFITIONS                           ///
////////////////////////////////////////////////////////////////////////

/**Function********************************************************************

  Synopsis    [Initializes the I/O package.]

  SideEffects []

  SeeAlso     [Io_End]

******************************************************************************/
void Ntki_Init( Mv_Frame_t * pMvsis )
{
    Cmd_CommandAdd( pMvsis, "Printing",  "print_stats",   Ntk_CommandPrintStats,       0 ); 
    Cmd_CommandAdd( pMvsis, "Printing",  "print",         Ntk_CommandPrint,            0 ); 
    Cmd_CommandAdd( pMvsis, "Printing",  "print_io",      Ntk_CommandPrintIo,          0 );
    Cmd_CommandAdd( pMvsis, "Printing",  "print_range",   Ntk_CommandPrintRange,       0 ); 
    Cmd_CommandAdd( pMvsis, "Printing",  "print_value",   Ntk_CommandPrintValue,       0 ); 
    Cmd_CommandAdd( pMvsis, "Printing",  "print_factor",  Ntk_CommandPrintFactor,      0 ); 
    Cmd_CommandAdd( pMvsis, "Printing",  "print_level",   Ntk_CommandPrintLevel,       0 ); 
    Cmd_CommandAdd( pMvsis, "Printing",  "print_nd",      Ntk_CommandPrintNd,          0 ); 
    Cmd_CommandAdd( pMvsis, "Printing",  "print_spec",    Ntk_CommandPrintSpec,        0 ); 

    Cmd_CommandAdd( pMvsis, "Printing",  "_print_clauses", Ntk_CommandPrintClauses,     0 ); 
    Cmd_CommandAdd( pMvsis, "Printing",  "_print_sat",     Ntk_CommandPrintSat,         0 ); 

    Cmd_CommandAdd( pMvsis, "Naming",    "reset_name",    Ntk_CommandResetName,        0 ); 
    Cmd_CommandAdd( pMvsis, "Naming",    "rename",        Ntk_CommandRename,           0 ); 
    Cmd_CommandAdd( pMvsis, "Naming",    "chng_name",     Ntk_CommandChangeNameMode,   0 ); 

    Cmd_CommandAdd( pMvsis, "Various",   "_power",         Ntk_CommandNetworkPower,     1 ); 
    Cmd_CommandAdd( pMvsis, "Various",   "_ndize",         Ntk_CommandNetworkNDize,     1 ); 
    Cmd_CommandAdd( pMvsis, "Various",   "dize",          Ntk_CommandNetworkDize,      1 ); 
    Cmd_CommandAdd( pMvsis, "Various",   "reorder",       Ntk_CommandNetworkReorder,   1 ); 
    Cmd_CommandAdd( pMvsis, "Various",   "_copy",          Ntk_CommandNetworkCopy,      1 ); 
    Cmd_CommandAdd( pMvsis, "Various",   "free",          Ntk_CommandNetworkFree,      1 ); 
    Cmd_CommandAdd( pMvsis, "Various",   "default",       Ntk_CommandNetworkDefault,   1 ); 
    Cmd_CommandAdd( pMvsis, "Various",   "window",        Ntk_CommandNetworkWindow,    0 ); 

    Cmd_CommandAdd( pMvsis, "Synthesis", "sweep",         Ntk_CommandNetworkSweep,     1 ); 
    Cmd_CommandAdd( pMvsis, "Synthesis", "eliminate",     Ntk_CommandNetworkEliminate, 1 ); 
    Cmd_CommandAdd( pMvsis, "Synthesis", "collapse",      Ntk_CommandNetworkCollapse,  1 ); 
    Cmd_CommandAdd( pMvsis, "Synthesis", "merge",         Ntk_CommandNetworkMerge,     1 ); 
    Cmd_CommandAdd( pMvsis, "Synthesis", "fxu",           Ntk_CommandNetworkFxu,       1 ); 
    Cmd_CommandAdd( pMvsis, "Synthesis", "decomp",        Ntk_CommandNetworkDecomp,    1 ); 
    Cmd_CommandAdd( pMvsis, "Synthesis", "strash",        Ntk_CommandNetworkStrash,    1 ); 
    Cmd_CommandAdd( pMvsis, "Synthesis", "reset_default", Ntk_CommandNetworkResetDef,  1 ); 
    Cmd_CommandAdd( pMvsis, "Synthesis", "_ffrsd",         Ntk_CommandNetworkFfResetDef,1 ); 
    Cmd_CommandAdd( pMvsis, "Synthesis", "resub",         Ntk_CommandNetworkResub,     1 ); 

    Cmd_CommandAdd( pMvsis, "Verification", "verify",     Ntk_CommandNetworkVerify,    0 ); 
}

/**Function********************************************************************

  Synopsis    [Ends the I/O package.]

  SideEffects []

  SeeAlso     [Io_Init]

******************************************************************************/
void Ntki_End( Mv_Frame_t * pMvsis )
{
}


/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandPrintStats( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    Ntk_Node_t * pNode;
    bool fComputeCvr;
    bool fComputeMvr;
    bool fShort;
    int nNodes, c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    // set the defaults
    fComputeCvr = 0;
    fComputeMvr = 0;
    fShort      = 1;
    nNodes = 0;
    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "scmh" ) ) != EOF )
    {
        switch ( c )
        {
        case 's':
            fShort ^= 1;
            break;
        case 'c':
            fComputeCvr ^= 1;
            break;
        case 'm':
            fComputeMvr ^= 1;
            break;
        case 'h':
            goto usage;
        default:
            goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( Mv_FrameReadErr(pMvsis), "Empty network\n" );
        return 1;
    }

    // collect the nodes (if any)
    if ( argc - util_optind > 0 )
	    nNodes = Cmd_CommandGetNodes( pMvsis, argc-util_optind+1, argv+util_optind-1, 0 );
    if ( nNodes == 0 ) 
    {
	    Ntk_NetworkPrintStats( pOut, pNet, fComputeCvr, fComputeMvr, fShort );
        return 0;
    }

    // print the nodes
    Ntk_NetworkForEachNodeSpecial( pNet, pNode )
	    Ntk_NodePrintStats( pOut, pNode, fComputeCvr, fComputeMvr );
    return 0;

usage:
    fprintf( pErr, "usage: print_stats [-c] [-m] [-h]\n" );
    fprintf( pErr, "\t           prints the network statistics and\n" );
    fprintf( pErr, "\t           reports the percentage of nodes having each representation\n" );
    fprintf( pErr, "\t-s       : output statistics in a shortened form [default = %s]\n", fShort? "yes": "no" );
    fprintf( pErr, "\t-c       : derives covers (MV SOPs) if not present [default = %s]\n", fComputeCvr? "yes": "no" );
    fprintf( pErr, "\t-m       : derives local MV relations if not present [default = %s]\n", fComputeMvr? "yes": "no" );
    fprintf( pErr, "\t-h       : print the command usage\n");
    return 1;
}

/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandPrint( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    Ntk_Node_t * pNode;
    int nNodes;
    int fPrintDefault;
    int fPrintPositional;
    int fPrintKmap;
    int c; 

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    fPrintDefault    = 0;
    fPrintPositional = 0;
    fPrintKmap       = 0;
    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "dpmh" ) ) != EOF )
    {
        switch ( c )
        {
        case 'd':
            fPrintDefault ^= 1;
            break;
        case 'p':
            fPrintPositional ^= 1;
            break;
        case 'm':
            fPrintKmap ^= 1;
            break;
        case 'h':
            goto usage;
            break;
        default:
            goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1; 
    }

    // collect the nodes
    nNodes = Cmd_CommandGetNodes( pMvsis, argc-util_optind+1, argv+util_optind-1, 0 );
    if ( nNodes == 0 )
    {
        fprintf( Mv_FrameReadErr(pMvsis), "The list of nodes to print is empty.\n" );
        return 1;
    }

    // print the nodes
    Ntk_NetworkForEachNodeSpecial( pNet, pNode )
        if ( fPrintKmap )
            Ntk_NodePrintMvr( pOut, pNode );
        else
            Ntk_NodePrintCvr( pOut, pNode, fPrintDefault, fPrintPositional );
    return 0;

usage:
    fprintf( pErr, "usage: print [-d] [-p] [-m] <node_list>\n" );
    fprintf( pErr, "\t               print MV SOP representation of the nodes.\n" );
    fprintf( pErr, "\t-d           : print default i-set [default = %s]\n", fPrintDefault? "yes": "no" );
    fprintf( pErr, "\t-p           : print cubes in positional notation [default = %s]\n", fPrintPositional? "yes": "no" );
    fprintf( pErr, "\t-m           : print Karnaugh maps for the relation [default = %s]\n", fPrintKmap? "yes": "no" );
    fprintf( pErr, "\t<node_list>  : if empty, print all nodes in the network\n" );
    fprintf( pErr, "\t-h           : print the command usage\n");
    return 1;
}

/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandPrintIo( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    int nNodes, c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    // set defaults
    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "h" ) ) != EOF )
    {
        switch ( c )
        {
        case 'h':
            goto usage;
        default:
            goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // collect the nodes
    if ( argc == util_optind )
        nNodes = 0;
    else
        nNodes = Cmd_CommandGetNodes( pMvsis, argc-util_optind+1, argv+util_optind-1, 1 );

    // print the nodes
    Ntk_NetworkPrintIo( pOut, pNet, nNodes );
    return 0;

usage:
    fprintf( pErr, "usage: print_io [-h] <node_list>\n" );
    fprintf( pErr, "\t               prints the fanins/fanouts of nodes in the list\n" );
    fprintf( pErr, "\t<node_list>  : if empty, print CIs/COs of the network\n" );
    fprintf( pErr, "\t-h           : print the command usage\n");
    return 1;
}

/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandPrintRange( Mv_Frame_t * pMvsis, int argc, char **argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    int nNodes, c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    // set defaults
    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "h" ) ) != EOF )
    {
        switch ( c )
        {
        case 'h':
            goto usage;
        default:
            goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // collect the nodes
    nNodes = Cmd_CommandGetNodes( pMvsis, argc-util_optind+1, argv+util_optind-1, 1 );
    if ( nNodes == 0 )
    {
        fprintf( pErr, "The list of nodes to print is empty.\n" );
        return 1;
    }

    // print the nodes
    Ntk_NetworkPrintRange( pOut, pNet, nNodes );
    return 0;

usage:
    fprintf( pErr, "usage: print_range [-h] <node_list>\n" );
    fprintf( pErr, "\t               prints the number of values of nodes in the list\n" );
    fprintf( pErr, "\t <node_list> : if empty, prints values for all nodes in the network\n" );
    fprintf( pErr, "\t-h           : print the command usage\n");
    return 1;
}

/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandPrintValue( Mv_Frame_t * pMvsis, int argc, char **argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    int nNodes, c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    // set defaults
    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "h" ) ) != EOF )
    {
        switch ( c )
        {
        case 'h':
            goto usage;
        default:
            goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // collect the nodes
    nNodes = Cmd_CommandGetNodes( pMvsis, argc-util_optind+1, argv+util_optind-1, 0 );
    if ( nNodes == 0 )
    {
        fprintf( pErr, "The list of nodes to print is empty.\n" );
        return 1;
    }

    // print the nodes
    Ntk_NetworkPrintValue( pOut, pNet, nNodes );
    return 0;

usage:
    fprintf( pErr, "usage: print_value [-h] <node_list>\n" );
    fprintf( pErr, "\t               prints the elimination value of nodes in the list\n" );
    fprintf( pErr, "\t <node_list> : if empty, prints values for all nodes in the network\n" );
    fprintf( pErr, "\t-h           : print the command usage\n");
    return 1;
}


/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandPrintFactor( Mv_Frame_t * pMvsis, int argc, char **argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    Ntk_Node_t * pNode;
    int nNodes, c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);
   
    // set defaults
    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "h" ) ) != EOF )
    {
        switch ( c )
        {
        case 'h':
            goto usage;
        default:
            goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // collect the nodes
    nNodes = Cmd_CommandGetNodes( pMvsis, argc-util_optind+1, argv+util_optind-1, 0 );
    if ( nNodes == 0 )
    {
        fprintf( pErr, "The list of nodes to print is empty.\n" );
        return 1;
    }

    // print the nodes
    Ntk_NetworkForEachNodeSpecial( pNet, pNode )
        Ntk_NodePrintFactor( pOut, pNode );
    return 0;

usage:
    fprintf( pErr, "usage: print_factor [-h] <node_list>\n" );
    fprintf( pErr, "\t     print algebraic factored form of the nodes.\n" );
    fprintf( pErr, "\t <node_list> : if empty, print all nodes in the network\n" );
    fprintf( pErr, "\t-h           : print the command usage\n");
    return 1;
}


/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandPrintLevel( Mv_Frame_t * pMvsis, int argc, char **argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    int fFromOutputs;
    int nNodes, c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);
   
    // set defaults
    fFromOutputs = 1;
    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "oh" ) ) != EOF )
    {
        switch ( c )
        {
            case 'o':
                fFromOutputs ^= 1;
                break;
            case 'h':
                goto usage;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // collect the nodes
    if ( argc == util_optind )
        nNodes = 0;
    else
        nNodes = Cmd_CommandGetNodes( pMvsis, argc-util_optind+1, argv+util_optind-1, 0 );
    
    // print nodes by level
    Ntk_NetworkPrintLevel( pOut, pNet, nNodes, fFromOutputs );
    return 0;

usage:
    fprintf( pErr, "usage: print_level [-o] [-h] <node_list>\n" );
    fprintf( pErr, "\t     prints nodes in the network by level.\n" );
    fprintf( pErr, "\t-o           : toggles DFS order used to levelize [default = %s]\n", fFromOutputs? "from COs": "from CIs");
    fprintf( pErr, "\t-h           : print the command usage\n");
    fprintf( pErr, "\t<node_list> : if empty, print all nodes in the network\n" );
    return 1;
}

/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandPrintNd( Mv_Frame_t * pMvsis, int argc, char **argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);
   
    // set defaults
    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "h" ) ) != EOF )
    {
        switch ( c )
        {
            case 'h':
                goto usage;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }
    
    // print nodes by level
    Ntk_NetworkPrintNd( pOut, pNet );
    return 0;

usage:
    fprintf( pErr, "usage: print_nd [-h]\n" );
    fprintf( pErr, "\t     prints the list of ND nodes of the current network\n" );
    fprintf( pErr, "\t-h          : print the command usage\n");
    return 1;
}


/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandPrintSpec( Mv_Frame_t * pMvsis, int argc, char **argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);
   
    // set defaults
    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "h" ) ) != EOF )
    {
        switch ( c )
        {
            case 'h':
                goto usage;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }
    
    // print nodes by level
    fprintf( pOut, "External spec file name is \"%s\".\n", pNet->pSpec );
    return 0;

usage:
    fprintf( pErr, "usage: print_spec [-h]\n" );
    fprintf( pErr, "\t        prints the external spec file name for the current network\n" );
    fprintf( pErr, "\t-h    : print the command usage\n");
    return 1;
}



/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandPrintClauses( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    char * pFileName;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "h" ) ) != EOF )
    {
        switch ( c )
        {
        case 'h':
            goto usage;
            break;
        default:
            goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // get the file name
    if ( util_optind < argc )
        pFileName = argv[util_optind];
    else
        pFileName = Extra_FileNameAppend(pNet->pName, ".cnf");

    // write the clauses
    Ntk_NetworkWriteClauses( pNet, pFileName ); 
    return 0;

usage:
    fprintf( pErr, "usage: _print_clauses <file_name>\n" );
    fprintf( pErr, "       generates MV SAT clauses for the network\n" );
    fprintf( pErr, "       <file_name>  (optional) the name of the output file.\n" );
    return 1;
}

/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandPrintSat( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    Ntk_Network_t * pNet2;
    char * pFileNameInput;
    char * pFileNameOutput;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "h" ) ) != EOF )
    {
        switch ( c )
        {
        case 'h':
            goto usage;
            break;
        default:
            goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // get the file name
    if ( util_optind >= argc )
    {
        fprintf( pErr, "The second file name is not given.\n" );
        goto usage;
    }

    // read the second network
    pFileNameInput  = argv[util_optind];
    pNet2 = Io_ReadNetwork( pMvsis, pFileNameInput );
   
    // get the output file name
    if ( util_optind+1 < argc )
        pFileNameOutput = argv[util_optind+1];
    else
    {
        char Buffer[100];
        sprintf( Buffer, "%s_%s", pNet->pName, pNet2->pName );
        pFileNameOutput = Extra_FileNameAppend(Buffer, ".mvsat");
    }

    Ntk_NetworkWriteSat( pNet, pNet2, pFileNameOutput ); 
    Ntk_NetworkDelete( pNet2 );
    return 0;

usage:
    fprintf( pErr, "usage: _print_sat <input_file> <output_file>\n" );
    fprintf( pErr, "       generates MV SAT clauses to express the non-equivalence\n" );
    fprintf( pErr, "       of the current network and the network found in <input_file>\n" );
    fprintf( pErr, "       (currently, this command works only for single-output networks)\n" );
    fprintf( pErr, "       <output_file>  (optional) the name of the output file.\n" );
    return 1;
}




/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandResetName( Mv_Frame_t * pMvsis, int argc, char **argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    int nNodesOld, nNodesNew, c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);
   
    // set defaults
    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "h" ) ) != EOF )
    {
        switch ( c )
        {
            case 'h':
                goto usage;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // print nodes by level
    nNodesOld = pNet->nIds;
    nNodesNew = Ntk_NetworkCompactNodeIds( pNet );
    fprintf( pOut, "The number of node IDs is reduced from %d to %d.\n", nNodesOld, nNodesNew );
    return 0;

usage:
    fprintf( pErr, "usage: reset_name [-h]\n" );
    fprintf( pErr, "\t     compacts node IDs resulting in shorter short names\n" );
    fprintf( pErr, "\t-h           : print the command usage\n");
    return 1;
}


/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandRename( Mv_Frame_t * pMvsis, int argc, char **argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);
   
    // set defaults
    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "h" ) ) != EOF )
    {
        switch ( c )
        {
            case 'h':
                goto usage;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // print nodes by level
    Ntk_NetworkRenameNodes( pNet );
    fprintf( pOut, "The names of all nodes have been changed.\n" );
    return 0;

usage:
    fprintf( pErr, "usage: rename [-h]\n" );
    fprintf( pErr, "\t     renames all nodes in the network\n" );
    fprintf( pErr, "\t-h     : print the command usage\n");
    return 1;
}


/**Function********************************************************************

  Synopsis    []

  Description []

  SideEffects []

  SeeAlso     []

******************************************************************************/
int Ntk_CommandChangeNameMode( Mv_Frame_t * pMvsis, int argc, char **argv )
{
    FILE * pOut, * pErr;

    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    if ( argc != 1 )
    {
        fprintf( pErr, "usage: chname\n" );
        fprintf( pErr, "       toggles between the short and long name modes\n" );
        return 1;
    }

    if ( Mv_FrameReadMode(pMvsis) )
    {
        Mv_FrameSetMode( pMvsis, 0 );
        fprintf( pOut, "Changing to long name mode.\n");
    }
    else 
    {
        Mv_FrameSetMode( pMvsis, 1 );
        fprintf( pOut, "Changing to short name mode.\n");
    }
    return 0;
}


/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandNetworkPower( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    int Degree, c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    Degree = 2;
    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "nh" ) ) != EOF )
    {
        switch ( c )
        {
            case 'n':
                if ( util_optind >= argc )
                {
                    fprintf( pErr, "Command line switch \"-n\" should be followed by an integer.\n" );
                    goto usage;
                }
                Degree = atoi(argv[util_optind]);
                util_optind++;
                if ( Degree < 0 ) 
                    goto usage;
                break;
            case 'h':
                goto usage;
                break;
            default:
            goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // write the clauses
    Ntk_NetworkComputePower( pNet, Degree ); 
    return 0;

usage:
    fprintf( pErr, "usage: _power [-n num]\n" );
    fprintf( pErr, "\t           generates the \"power\" of the given network\n" );
    fprintf( pErr, "\t-n num   : the number of levels [default = %d].\n", Degree );
    return 1;
}

/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandNetworkNDize( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    double Prob;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    Prob = 0.1;
    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "ph" ) ) != EOF )
    {
        switch ( c )
        {
        case 'p':
            Prob = atof(argv[util_optind]);
            util_optind++;
            if ( Prob < 0 ) 
                goto usage;
            break;
        case 'h':
            goto usage;
            break;
        default:
            goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // write the clauses
    Ntk_NetworkNonDeterminize( pNet, Prob ); 
    return 0;

usage:
    fprintf( pErr, "usage: _ndize [-p num] [-h]\n" );
    fprintf( pErr, "\t          non-determinizes the network by adding ND buffers\n" );
    fprintf( pErr, "\t-p num  : a floating point number (0 <= num <= 1) [default = %.2f].\n", Prob );
    return 1;
}

/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandNetworkDize( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    Ntk_Node_t * pNode;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "h" ) ) != EOF )
    {
        switch ( c )
        {
        case 'h':
            goto usage;
            break;
        default:
            goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // write the clauses
    Ntk_NetworkForEachNode( pNet, pNode )
        Ntk_NodeDeterminize( pNode );

//    Ntk_NetworkEncode( pNet );
    return 0;

usage:
    fprintf( pErr, "usage: dize [-h]\n" );
    fprintf( pErr, "\t          determinizes ND nodes in the network\n" );
    return 1;
}

/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandNetworkReorder( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    int c, fVerbose = 0;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "vh" ) ) != EOF )
    {
        switch ( c )
        {
            case 'v':
                fVerbose ^= 1;
                break;
            case 'h':
                goto usage;
                break;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // reorder the local function of each node
    Ntk_NetworkReorder( pNet, fVerbose );
    return 0;

usage:
    fprintf( pErr, "usage: reorder [-v]\n" );
    fprintf( pErr, "\t         reorders BDDs of the local MV relations of the nodes\n" );
    fprintf( pErr, "\t-v     : enable printing out statistics [default = %d].\n", fVerbose? "yes": "no" );  
    return 1;
}

/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
Ntk_CommandNetworkCopy( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    int fVerbose;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    // set the defaults
    fVerbose = 0;
    util_getopt_reset();
    while ( (c = util_getopt(argc, argv, "hv")) != EOF ) 
    {
        switch (c) 
        {
            case 'v':
                fVerbose ^= 1;
                break;
            case 'h':
                goto usage;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // replace the current network without modifying the stack of saved networks
    Mv_FrameReplaceCurrentNetwork( pMvsis, Ntk_NetworkDup(pNet, pNet->pMan) );
    return 0;

usage:
    fprintf( pErr, "usage: copy [-h]\n");
    fprintf( pErr, "\t         replaces the current network by its copy\n" );  
    fprintf( pErr, "\t         does not modify the stack of the saved networks\n" );  
    fprintf( pErr, "\t-h     : print the command usage\n");
    return 1;       /* error exit */
}


/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
Ntk_CommandNetworkFree( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    int fVerbose;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    // set the defaults
    fVerbose = 0;
    util_getopt_reset();
    while ( (c = util_getopt(argc, argv, "hv")) != EOF ) 
    {
        switch (c) 
        {
            case 'v':
                fVerbose ^= 1;
                break;
            case 'h':
                goto usage;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // get the value
    if ( util_optind < argc )
    {
        Ntk_Node_t * pNode;
        if ( strcmp( argv[util_optind], "cvr" ) == 0 )
        {
            Ntk_NetworkForEachNode( pNet, pNode )
            {
                Ntk_NodeGetFuncMvr( pNode );
                Ntk_NodeFreeFuncCvr( pNode );
            }
            return 0;
        }
        else if ( strcmp( argv[util_optind], "mvr" ) == 0 )
        {
            Ntk_NetworkForEachNode( pNet, pNode )
            {
                Ntk_NodeGetFuncCvr( pNode );
                Ntk_NodeFreeFuncMvr( pNode );
            }
            return 0;
        }
    }

    fprintf( pOut, "The object to free is not specified\n" );  
    return 0;

usage:
    fprintf( pErr, "usage: free <object> [-h]\n");
    fprintf( pErr, "\t           frees MV SOPs (\"cvr\") or MV relations (\"mvr\") in all nodes\n" );  
    fprintf( pErr, "\t<object> : the representation to free (\"cvr\" or \"mvr\")\n" );
    fprintf( pErr, "\t-h       : print the command usage\n");
    return 1;       /* error exit */
}

/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
Ntk_CommandNetworkDefault( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    bool fVerbose;
    bool fUseDefault;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    // set the defaults
    fVerbose = 0;
    fUseDefault = 1;
    util_getopt_reset();
    while ( (c = util_getopt(argc, argv, "dhv")) != EOF ) 
    {
        switch (c) 
        {
            case 'd':
                fUseDefault ^= 1;
                break;
            case 'v':
                fVerbose ^= 1;
                break;
            case 'h':
                goto usage;
                break;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // make sure the default is always present whenever possible
    if ( fUseDefault )
        Ntk_NetworkForceDefault( pNet );
    else
        Ntk_NetworkComputeDefault( pNet );
    return 0;

usage:
    fprintf( pErr, "usage: default [-d] [-h]\n");
    fprintf( pErr, "\t         adds/removes the default covers in determistic nodes\n" );  
    fprintf( pErr, "\t-d     : toggles adding/removing the default value [default = %s]\n", fUseDefault? "remove": "add" );
    fprintf( pErr, "\t-h     : print the command usage\n");
    return 1;       /* error exit */
}

/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
Ntk_CommandNetworkWindow( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    char pFileName[100];
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    Ntk_Node_t * pNode;
    int nNodes;
    int nTfiLevels;
    int nTfoLevels;
    int fVerbose;
    int fPrintAll;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);
 
    // set the defaults
    nTfiLevels = 2;
    nTfoLevels = 2;
    fVerbose   = 0;
    fPrintAll  = 0;
    util_getopt_reset();
    while ( (c = util_getopt(argc, argv, "ioahv")) != EOF ) 
    {
        switch (c) 
        {
            case 'i':
                if ( util_optind >= argc )
                {
                    fprintf( pErr, "Command line switch \"-i\" should be followed by an integer.\n" );
                    goto usage;
                }
                nTfiLevels = atoi(argv[util_optind]);
                util_optind++;
                if ( nTfiLevels < 0 ) 
                    goto usage;
                break;
            case 'o':
                if ( util_optind >= argc )
                {
                    fprintf( pErr, "Command line switch \"-o\" should be followed by an integer.\n" );
                    goto usage;
                }
                nTfoLevels = atoi(argv[util_optind]);
                util_optind++;
                if ( nTfoLevels < 0 ) 
                    goto usage;
                break;
            case 'a':
                fPrintAll ^= 1;
                break;
            case 'v':
                fVerbose ^= 1;
                break;
            case 'h':
                goto usage;
                break;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    if ( fPrintAll )
    {
        Ntk_NetworkForEachNode( pNet, pNode )
        {
            Wn_Window_t * pWnd, * pWndCore;
            // create the core window from the linked list of node in pNet
            pWndCore = Wn_WindowCreateFromNode( pNode );
            // derive the container window for the core window
            pWnd     = Wn_WindowDerive( pWndCore, nTfiLevels, nTfoLevels );
            // print the stats
            fprintf( Ntk_NetworkReadMvsisOut(pNet), "%5s  fi = %3d  fo = %3d :  lvs = %5d  rts = %5d  nds = %5d\n", 
                   Ntk_NodeGetNamePrintable(pNode), 
                   Ntk_NodeReadFaninNum(pNode),
                   Ntk_NodeReadFanoutNum(pNode),
                   Wn_WindowReadLeafNum(pWnd),
                   Wn_WindowReadRootNum(pWnd),
                   Wn_WindowReadNodeNum(pWnd) );
            // delete the window
            Wn_WindowDelete( pWnd );
        }
        return 0;
    }

    if ( util_optind == argc )
    {
        fprintf( pErr, "The list of nodes for extracting is not given.\n" );
        return 1;
    }

    // collect the nodes
    nNodes = Cmd_CommandGetNodes( pMvsis, argc-util_optind+1, argv+util_optind-1, 0 );
    if ( nNodes == 0 )
    {
        fprintf( pErr, "The list of nodes to be extracted is empty.\n" );
        return 1;
    }
    pNode = pNet->pOrder;

    // the nodes to be merged are linked into the special linked list
    if ( nTfiLevels > 0 || nTfoLevels > 0 )
    {
        Wn_Window_t * pWnd, * pWndCore;
        // create the core window from the linked list of node in pNet
        pWndCore = Wn_WindowCreateFromNodeArray( pNet );
        // derive the container window for the core window
        pWnd     = Wn_WindowDerive( pWndCore, nTfiLevels, nTfoLevels );
        // collect the internal nodes of the container window
        Wn_WindowCollectInternal( pWnd );
        // put the nodes into the list
        Ntk_NetworkCreateSpecialFromArray( pNet, Wn_WindowReadNodes(pWnd), Wn_WindowReadNodeNum(pWnd) );
        // delete the window
        Wn_WindowDelete( pWnd );
    }
    // create the name for the new file
    if ( Ntk_NetworkIsBinary(pNet) )
    {
        sprintf( pFileName, "%s_%d_%s.blif", pNet->pName, nNodes, 
            Ntk_NodeGetNamePrintable(pNode) );
        Ntk_SubnetworkWriteIntoFile( pNet, pFileName, 1 );
    }
    else
    {
        sprintf( pFileName, "%s_%d_%s.mv", pNet->pName, nNodes, 
            Ntk_NodeGetNamePrintable(pNode) );
        Ntk_SubnetworkWriteIntoFile( pNet, pFileName, 0 );
    }
    return 0;

usage:
    fprintf( pErr, "usage: window [-i num] [-o num] [-a] [-h] <node_list>\n");
    fprintf( pErr, "\t         carves out a window surrounding nodes in the list\n" );  
    fprintf( pErr, "\t-i num : the number of levels of the limited TFI [default = %d]\n", nTfiLevels );  
    fprintf( pErr, "\t-o num : the number of levels of the limited TFO [default = %d]\n", nTfoLevels );  
    fprintf( pErr, "\t-a     : print statistics about all windows [default = %s]\n", fPrintAll? "yes": "no" );
    fprintf( pErr, "\t-h     : print the command usage\n");
    return 1;       /* error exit */
}


/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
Ntk_CommandNetworkSweep( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    int fSweepConsts;
    int fSweepBuffers;
    int fSweepIsets;
    int fVerbose;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    // set the defaults
    fSweepConsts  = 1;
    fSweepBuffers = 1;
    fSweepIsets   = 0;
    fVerbose      = 0;
    util_getopt_reset();
    while ( (c = util_getopt(argc, argv, "cbivh")) != EOF ) 
    {
        switch (c) 
        {
            case 'c':
                fSweepConsts ^= 1;
                break;
            case 'b':
                fSweepBuffers ^= 1;
                break;
            case 'i':
                fSweepIsets ^= 1;
                break;
            case 'v':
                fVerbose ^= 1;
                break;
            case 'h':
                goto usage;
                break;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    Ntk_NetworkSweep( pNet, fSweepConsts, fSweepBuffers, fSweepIsets, fVerbose );
    return 0;

usage:
    fprintf( pErr, "usage: sweep [-b] [-i] [-v] [-h]\n");
    fprintf( pErr, "\t         simplifies the network by removing useless nodes\n");
    fprintf( pErr, "\t-c     : toggle removing the constant nodes [default = %s]\n", fSweepConsts? "yes": "no" );  
    fprintf( pErr, "\t-b     : toggle removing the single-input nodes [default = %s]\n", fSweepBuffers? "yes": "no" );  
    fprintf( pErr, "\t-i     : toggle sweeping i-sets of the internal nodes [default = %s]\n", fSweepIsets? "yes": "no" );  
    fprintf( pErr, "\t-v     : print verbose information [default = %s]\n", fVerbose? "yes": "no" );  
    fprintf( pErr, "\t-h     : print the command usage\n");
    return 1;       /* error exit */
}

/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
Ntk_CommandNetworkEliminate( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    bool fVerbose;
    bool fUseCostBdd;
    int Value, nNodesElim, c;
    int nCubeLimit;
    int nFaninLimit;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    // set the defaults
    nNodesElim  = 100000;
    nCubeLimit  = 500;
    nFaninLimit = 200;
    Value       = -1;
    fUseCostBdd = 0;
    fVerbose    = 0;

    util_getopt_reset();
    while ( (c = util_getopt(argc, argv, "cflbvh")) != EOF ) 
    {
        switch (c) 
        {
            case 'c':
                if ( util_optind >= argc )
                {
                    fprintf( pErr, "Command line switch \"-c\" should be followed by an integer.\n" );
                    goto usage;
                }
                nCubeLimit = atoi(argv[util_optind]);
                util_optind++;
                if ( nCubeLimit < 0 ) 
                    goto usage;
                break;
            case 'f':
                if ( util_optind >= argc )
                {
                    fprintf( pErr, "Command line switch \"-f\" should be followed by an integer.\n" );
                    goto usage;
                }
                nFaninLimit = atoi(argv[util_optind]);
                util_optind++;
                if ( nFaninLimit < 0 ) 
                    goto usage;
                break;
            case 'l':
                if ( util_optind >= argc )
                {
                    fprintf( pErr, "Command line switch \"-l\" should be followed by an integer.\n" );
                    goto usage;
                }
                nNodesElim = atoi(argv[util_optind]);
                util_optind++;
                if ( nNodesElim < 0 ) 
                    goto usage;
                break;
            case 'b':
                fUseCostBdd ^= 1;
                break;
            case 'v':
                fVerbose ^= 1;
                break;
            case 'h':
                goto usage;
                break;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    if ( util_optind == argc )
    {
        fprintf( pErr, "Elimination value is not given; %d is assumed.\n", Value );
    }
    else if ( util_optind == argc - 1 )
    {
        Value = atoi(argv[util_optind]);
        if ( Value < -1 )
        {
            fprintf( pErr, "Elimination value is incorrect (%d).\n", Value );
            goto usage;
        }
    }
    else
        goto usage;

    // call the elimination procedure
    Ntk_NetworkEliminate( pNet, nNodesElim, nCubeLimit, nFaninLimit, -Value, fUseCostBdd, fVerbose );
    return 0;

usage:
    fprintf( pErr, "usage: eliminate [-c num] [-f num] [-l num] [-b] [-v] [-h] <value>\n" );
    fprintf( pErr, "\t         selectively collapses the nodes into their fanouts\n" );  
    fprintf( pErr, "\t-c num : the cube limit (if more, skip the node) [default = %d]\n", nCubeLimit );
    fprintf( pErr, "\t-f num : the fanin limit (if more, skip the node) [default = %d]\n", nFaninLimit );
    fprintf( pErr, "\t-l num : the maximum number of nodes to eliminate [default = %d]\n", nNodesElim );
    fprintf( pErr, "\t-b     : use BDDs nodes instead of FF literals as a cost [default = %s]\n", fUseCostBdd? "yes": "no" );  
    fprintf( pErr, "\t-v     : prints verbose information about elimination [default = %s]\n", fVerbose? "yes": "no" );  
    fprintf( pErr, "\t-h     : print the command usage\n");
    fprintf( pErr, "\t<value>: the elimination value [default = %d]\n", Value );
    return 1;       /* error exit */
}


/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
Ntk_CommandNetworkCollapse( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet, * pNetNew;
    Ntk_Node_t * pNode1, * pNode2;
    int c;
    int fVerbose;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    // set the defaults
    fVerbose = 0;
    util_getopt_reset();
    while ( (c = util_getopt(argc, argv, "hv")) != EOF ) 
    {
        switch (c) 
        {
            case 'h':
                goto usage;
                break;
            case 'v':
                fVerbose ^= 1;
                break;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    if ( argc == util_optind + 2 ) 
    {
        pNode1  = Ntk_NetworkCollectNodeByName( pNet, argv[util_optind],   0 );
        pNode2  = Ntk_NetworkCollectNodeByName( pNet, argv[util_optind+1], 0 );
        if ( !pNode1 || !pNode2 )
            return 1;

        if ( Ntk_NodeReadFaninIndex( pNode1, pNode2 ) != -1 )
            Ntk_NetworkCollapseNodes( pNode1, pNode2 );
        else if ( Ntk_NodeReadFaninIndex( pNode2, pNode1 ) != -1 )
            Ntk_NetworkCollapseNodes( pNode2, pNode1 );
        else
        {
            fprintf( pErr, "Nodes \"%s\" and \"%s\" are not fanin/fanout of each other.\n", 
                Ntk_NodeGetNamePrintable(pNode1), Ntk_NodeGetNamePrintable(pNode2) );
            return 1;
        }
    }
    else if ( argc == util_optind ) 
    {
        // single level networks cannot be collapsed any further
        if ( Ntk_NetworkGetNumLevels(pNet) < 2 )
	        return 0;

        // perform collapsing of the network
        pNetNew = Ntk_NetworkCollapse( pNet, fVerbose );
        if ( pNetNew == NULL )
        {
            fprintf( pErr, "Network collapse has failed!\n" );
            return 1;
        }
        // replace the current network
        Mv_FrameReplaceCurrentNetwork( pMvsis, pNetNew );
    }
    else if ( argc == util_optind + 1 ) 
    {

        pNode1  = Ntk_NetworkCollectNodeByName( pNet, argv[util_optind],   0 );
        if ( !pNode1  )
            return 1;
        Ntk_NetworkCollapseNode( pNet, pNode1 );
    }
    else
    {
        fprintf( pErr, "Incorrect number of command line arguments.\n" );
        return 1;
    }

    return 0;

usage:
    fprintf( pErr, "usage: collapse [-v] [-h] <node1> <node2>\n");
    fprintf( pErr, "\t          collapses the nodes or the network (default)\n" );  
    fprintf( pErr, "\t<node1> : the node to collapse into its fanouts\n");
    fprintf( pErr, "\t<node2> : if given, the node to collapse into <node1>\n");
    fprintf( pErr, "\t-v      : toggles verbose output [default = %s]\n", fVerbose? "yes": "no" );  
    fprintf( pErr, "\t-h      : print the command usage\n");
    return 1;       /* error exit */
}


/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
Ntk_CommandNetworkMerge( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    int nNodes;
    int fVerbose;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    // set the defaults
    fVerbose = 0;
    util_getopt_reset();
    while ( (c = util_getopt(argc, argv, "hv")) != EOF ) 
    {
        switch (c) 
        {
            case 'v':
                fVerbose ^= 1;
                break;
            case 'h':
                goto usage;
                break;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    if ( util_optind == argc )
    {
        fprintf( pErr, "The list of nodes for merging is not given.\n" );
        return 1;
    }

    // collect the nodes
    nNodes = Cmd_CommandGetNodes( pMvsis, argc-util_optind+1, argv+util_optind-1, 0 );
    if ( nNodes < 2 )
    {
        fprintf( pErr, "The list of nodes contains less then two internal nodes.\n" );
        return 1;
    }

    // the nodes to be merged are linked into the special linked list
    Ntk_NetworkMerge( pNet, nNodes );
    return 0;

usage:
    fprintf( pErr, "usage: merge <node_list>\n");
    fprintf( pErr, "\t         merges all the nodes in the list\n" );  
    fprintf( pErr, "\t-h     : print the command usage\n");
    return 1;       /* error exit */
}


/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
Ntk_CommandNetworkFxu( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    Ntk_Network_t * pNet;
    FILE * pOut, * pErr;
    Fxu_Data_t * p = NULL;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    // allocate the structure
    p = ALLOC( Fxu_Data_t, 1 );
    memset( p, 0, sizeof(Fxu_Data_t) );
    // set the defaults
    p->nPairsMax = 30000;
    p->nNodesExt = 10000;
    p->fOnlyS    = 0;
    p->fOnlyD    = 0;
    p->fUse0     = 0;
    p->fUseCompl = 1;
    p->fVerbose  = 0;
    util_getopt_reset();
    while ( (c = util_getopt(argc, argv, "lnsdzcvh")) != EOF ) 
    {
        switch (c) 
        {
            case 'l':
                if ( util_optind >= argc )
                {
                    fprintf( pErr, "Command line switch \"-l\" should be followed by an integer.\n" );
                    goto usage;
                }
                p->nPairsMax = atoi(argv[util_optind]);
                util_optind++;
                if ( p->nPairsMax < 0 ) 
                    goto usage;
                break;
            case 'n':
                if ( util_optind >= argc )
                {
                    fprintf( pErr, "Command line switch \"-n\" should be followed by an integer.\n" );
                    goto usage;
                }
                p->nNodesExt = atoi(argv[util_optind]);
                util_optind++;
                if ( p->nNodesExt < 0 ) 
                    goto usage;
                break;
            case 's':
                p->fOnlyS ^= 1;
                break;
            case 'd':
                p->fOnlyD ^= 1;
                break;
            case 'z':
                p->fUse0 ^= 1;
                break;
            case 'c':
                p->fUseCompl ^= 1;
                break;
            case 'v':
                p->fVerbose ^= 1;
                break;
            case 'h':
                goto usage;
                break;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        Ntk_NetworkFxFreeInfo( p );
        return 1;
    }

    // the nodes to be merged are linked into the special linked list
    Ntk_NetworkFastExtract( pNet, p );
    Ntk_NetworkFxFreeInfo( p );
    return 0;

usage:
    fprintf( pErr, "usage: fxu [-n num] [-l num] [-s] [-d] [-z] [-c] [-v] [-h]\n");
    fprintf( pErr, "\t         performs unate fast extract on the current network\n");
    fprintf( pErr, "\t-n num : the maximum number of divisors to extract [default = %d]\n", p->nNodesExt );  
    fprintf( pErr, "\t-l num : the maximum number of cube pairs to consider [default = %d]\n", p->nPairsMax );  
    fprintf( pErr, "\t-s     : use only single-cube divisors [default = %s]\n", p->fOnlyS? "yes": "no" );  
    fprintf( pErr, "\t-d     : use only double-cube divisors [default = %s]\n", p->fOnlyD? "yes": "no" );  
    fprintf( pErr, "\t-z     : use zero-weight divisors [default = %s]\n", p->fUse0? "yes": "no" );  
    fprintf( pErr, "\t-c     : use complement in the binary case [default = %s]\n", p->fUseCompl? "yes": "no" );  
    fprintf( pErr, "\t-v     : print verbose information [default = %s]\n", p->fVerbose? "yes": "no" ); 
    fprintf( pErr, "\t-h     : print the command usage\n");
    Ntk_NetworkFxFreeInfo( p );
    return 1;       
}

/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
Ntk_CommandNetworkDecomp( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    int fVerbose;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    // set the defaults
    fVerbose = 0;
    util_getopt_reset();
    while ( (c = util_getopt(argc, argv, "hv")) != EOF ) 
    {
        switch (c) 
        {
            case 'v':
                fVerbose ^= 1;
                break;
            case 'h':
                goto usage;
                break;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    Ntk_NetworkDecomp( pNet );
    return 0;

usage:
    fprintf( pErr, "usage: decomp [-h]\n");
    fprintf( pErr, "\t         decomposes the network using the factored forms\n" );  
    fprintf( pErr, "\t-h     : print the command usage\n");
    return 1;       /* error exit */
}

/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
int Ntk_CommandNetworkStrash( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet, * pNetNew;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    util_getopt_reset();
    while ( ( c = util_getopt( argc, argv, "h" ) ) != EOF )
    {
        switch ( c )
        {
            case 'h':
                goto usage;
                break;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // create the structurally hashed network
    pNetNew = Ntk_NetworkStrash( pNet );
    if ( pNetNew == NULL )
    {
        fprintf( pErr, "Network strashing has failed!\n" );
        return 1;
    }

    // replace the current network
    Mv_FrameReplaceCurrentNetwork( pMvsis, pNetNew );
    return 0;

usage:
    fprintf( pErr, "usage: strash\n" );
    fprintf( pErr, "\t     replaces the current network by SS-equivalent\n" );
    fprintf( pErr, "\t     structurally hashed network of AND/INV gates\n" );
    fprintf( pErr, "\t-h     : print the command usage\n");
    return 1;
}


/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
Ntk_CommandNetworkResetDef( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    Ntk_Node_t * pNode;
    int fVerbose;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    // set the defaults
    fVerbose = 0;
    util_getopt_reset();
    while ( (c = util_getopt(argc, argv, "hv")) != EOF ) 
    {
        switch (c) 
        {
            case 'v':
                fVerbose ^= 1;
                break;
            case 'h':
                goto usage;
                break;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // get the node name
    pNode = NULL;
    if ( util_optind < argc )
    {
        pNode = Ntk_NetworkCollectNodeByName( pNet, argv[util_optind], 0 );
        if ( pNode == NULL )
            goto usage;
        Ntk_NodeResetDefault( pNode, Ntk_NetworkGetAcceptType(pNet) );
        return 0;
    }

    Ntk_NetworkResetDefault( pNet );
    return 0;

usage:
    fprintf( pErr, "usage: reset_default [-h] <node>\n");
    fprintf( pErr, "\t         assigns the default values by minimizing MV relations\n" );  
    fprintf( pErr, "\t-h     : print the command usage\n");
    fprintf( pErr, "\t<node> : the name of one node to reset default\n");
    return 1;       /* error exit */
}


/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
Ntk_CommandNetworkFfResetDef( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    Ntk_Node_t * pNode;
    int fVerbose;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    // set the defaults
    fVerbose = 0;
    util_getopt_reset();
    while ( (c = util_getopt(argc, argv, "hv")) != EOF ) 
    {
        switch (c) 
        {
            case 'v':
                fVerbose ^= 1;
                break;
            case 'h':
                goto usage;
                break;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // get the node name
    pNode = NULL;
    if ( util_optind < argc )
    {
        pNode = Ntk_NetworkCollectNodeByName( pNet, argv[util_optind], 0 );
        if ( pNode == NULL )
            goto usage;
        Ntk_NodeFfResetDefault( pNode, 0, fVerbose );
        return 0;
    }


    Ntk_NetworkFfResetDefault( pNet, fVerbose );
    return 0;

usage:
    fprintf( pErr, "usage: _ffrsd [-h] <node>\n");
    fprintf( pErr, "\t         phase-assigns binary covers using FF complement\n" );  
    fprintf( pErr, "\t-h     : print the command usage\n");
    fprintf( pErr, "\t<node> : the name of one node to reset default\n");
    return 1;       /* error exit */
}

/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
Ntk_CommandNetworkResub( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    FILE * pOut, * pErr;
    Ntk_Network_t * pNet;
    Ntk_Node_t * pNode; 
    int nCands;
    bool fVerbose;
    int c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    fVerbose = 0;
    nCands   = 8;
    util_getopt_reset();
    while ( (c = util_getopt(argc, argv, "cvh")) != EOF ) 
    {
        switch (c) 
        {
            case 'c':
                if ( util_optind >= argc )
                {
                    fprintf( pErr, "Command line switch \"-c\" should be followed by an integer.\n" );
                    goto usage;
                }
                nCands = atoi(argv[util_optind]);
                util_optind++;
                if ( nCands < 0 ) 
                    goto usage;
                break;
            case 'v':
                fVerbose ^= 1;
                break;
            case 'h':
                goto usage;
                break;
            default:
                goto usage;
        }
    }

    if ( pNet == NULL )
    {
        fprintf( pErr, "Empty network.\n" );
        return 1;
    }

    // get the node name
    if ( util_optind < argc )
    {
        pNode = Ntk_NetworkCollectNodeByName( pNet, argv[util_optind], 0 );
        if ( pNode == NULL )
            goto usage;
        Ntk_NetworkResubNode( pNode, Ntk_NodeGetFuncMvr(pNode), Ntk_NetworkGetAcceptType(pNet), nCands, fVerbose );
        return 0;
    }

    Ntk_NetworkResub( pNet, nCands, fVerbose );
    return 0;

usage:
    fprintf( pErr, "usage: resub [-c num] [-v] [-h] <name>\n");
    fprintf( pErr, "\t         performs boolean resubstitution on the network\n" );  
    fprintf( pErr, "\t-c num : the max number of resubstitution candidates [default = %d]\n", nCands ); 
    fprintf( pErr, "\t-v     : print verbose information [default = %s]\n", fVerbose? "yes": "no" ); 
    fprintf( pErr, "\t-h     : print the command usage\n");
    fprintf( pErr, "\t<name> : the name of one node to resubstitute into\n");
    return 1;       /* error exit */
}


/**Function*************************************************************

  Synopsis    []

  Description []
               
  SideEffects []

  SeeAlso     []

***********************************************************************/
Ntk_CommandNetworkVerify( Mv_Frame_t * pMvsis, int argc, char ** argv )
{
    Ntk_Network_t * pNet, * pNet1, * pNet2;
    FILE * pOut, * pErr, * pFile;
    int fVerbose, c;

    pNet = Mv_FrameReadNet(pMvsis);
    pOut = Mv_FrameReadOut(pMvsis);
    pErr = Mv_FrameReadErr(pMvsis);

    // set the defaults
    fVerbose = 0;
    util_getopt_reset();
    while ((c = util_getopt(argc, argv, "vh")) != EOF) 
    {
        switch(c) 
        {
            case 'v':
                fVerbose ^= 1;
                break;
            case 'h':
                goto usage;
            default:
                goto usage;
        }
    }

    if ( argc == util_optind ) 
    { // use the spec
        if ( pNet == NULL )
        {
            fprintf( pErr, "Empty current network.\n" );
            return 1;
        }
        if ( pNet->pSpec == NULL )
        {
            fprintf( pErr, "The external spec is not given; cannot perform verification.\n" );
            return 1;
        }

        pFile = fopen( pNet->pSpec, "r" );
        if ( pFile == NULL )
        {
            fprintf( pErr, "Cannot open the external spec file \"%s\"; cannot perform verification.\n", pNet->pSpec );
            return 1;
        }
        else
            fclose( pFile );

        // the first network is the current network
        pNet1 = pNet;
        // read the spec network using the same functionality manager
        pNet2 = Io_ReadNetwork( pMvsis, pNet->pSpec );
        if ( pNet2 == NULL )
            return 1;
        // verify
        Ntk_NetworkVerify( pNet1, pNet2, fVerbose );
        // delete the second network
        Ntk_NetworkDelete( pNet2 );
        return 0;
    }
    if ( argc == util_optind + 1 ) 
    {
        if ( pNet == NULL )
        {
            fprintf( pErr, "Empty current network.\n" );
            return 1;
        }
        // the first network is the current network
        pNet1 = pNet;
        // read the second network using the same functionality manager
        pNet2 = Io_ReadNetwork( pMvsis, argv[util_optind] );
        if ( pNet2 == NULL )
            return 1;
        // verify
        Ntk_NetworkVerify( pNet1, pNet2, fVerbose );
        // delete the second network
        Ntk_NetworkDelete( pNet2 );
        return 0;
    }
    if ( argc == util_optind + 2 ) 
    {
        Fnc_Manager_t * pManOld, * pManNew;

        // perform verification in the new functionality manager
        pManNew = Fnc_ManagerAllocate();
        pManOld = Mv_FrameSetMan( pMvsis, pManNew );

        // read the first network with the new functionality manager
        pNet1 = Io_ReadNetwork( pMvsis,  argv[util_optind] );
        if ( pNet1 == NULL )
            return 1;
        // read the second network with the same functionality manager
        pNet2 = Io_ReadNetwork( pMvsis, argv[util_optind+1] );
        if ( pNet2 == NULL )
        {
            Ntk_NetworkDelete( pNet1 );
            return 1;
        }
        // verify
        Ntk_NetworkVerify( pNet1, pNet2, fVerbose );
        // delete the network (should also delete the new functionality manager)
        Ntk_NetworkDelete( pNet1 );
        Ntk_NetworkDelete( pNet2 );

        // set the old manager
        pManNew = Mv_FrameSetMan( pMvsis, pManOld );
        // deallocate the new manager
        Fnc_ManagerDeallocate( pManNew );
        return 0;
    }

usage:
    fprintf( pErr, "usage: verify [-v] [-h] <file1> <file2>\n");
    fprintf( pErr, "\t          verifies the containment of two networks\n" );  
    fprintf( pErr, "\t-v      : print debug information [default = %s]\n", fVerbose? "yes": "no" );  
    fprintf( pErr, "\t-h      : print the command usage\n");
    fprintf( pErr, "\t<file1> : (if given) the network to verify against the current network\n" );  
    fprintf( pErr, "\t<file2> : (if given) the network to verify against <file1>\n" );  
    fprintf( pErr, "\t        : if no file name is given, verifies against the spec\n" );  
    return 1;       /* error exit */
}


////////////////////////////////////////////////////////////////////////
///                       END OF FILE                                ///
////////////////////////////////////////////////////////////////////////


